#include "Frame.h"
#include "Graphics.h"
#include <wx/sizer.h>
#include <wx/panel.h>
#include <wx/stattext.h>
#include <wx/msgdlg.h>

#define ID_SCLTILES			0
#define ID_SCLPALETTE		1
#define ID_SCLTILEMAP		2
#define ID_BTNSELECT		10
#define ID_BTNOPENTILES		20
#define ID_BTNOPENPALETTE	21
#define ID_BTNOPENPALETTE24	22
#define ID_CMBBITSIZE		50

wxBEGIN_EVENT_TABLE(MyFrame, wxFrame)
	EVT_SHOW(MyFrame::OnShow)
	EVT_COMMAND_RANGE(ID_BTNOPENTILES, ID_BTNOPENPALETTE24, wxEVT_BUTTON, MyFrame::OnButtonOpen)
	/*EVT_MENU_RANGE(wxID_OPEN, wxID_EXIT, MyFrame::OnMenu)*/
	EVT_COMBOBOX(ID_CMBBITSIZE, MyFrame::OnBitSizeUpdate)
wxEND_EVENT_TABLE()

void MyFrame::OnShow(wxShowEvent &e) {
	if (e.IsShown() == false) return;
	wxMessageBox("First, load the palette in the lower left; if you have the SFC-formatted palette,"
		"choose the top button; otherwise, for a 24-bit formatted palette, choose the bottom button.\n\n"
		"After that, choose the tileset in the upper left.\n\n"
		"And then finally, go to File->Open to open the tile map.",
		"To do", wxOK, this);
}

void MyFrame::OnBitSizeUpdate(wxCommandEvent& e) {
	sclTiles->SetBitSize(cmbBitSize->GetCurrentSelection());
}

void MyFrame::OnButtonOpen(wxCommandEvent &e) {
	switch (e.GetId()) {
	case ID_BTNOPENPALETTE24: 
		sclPalette->Load24BitRGBPalette();
		break;
	case ID_BTNOPENPALETTE: 
		sclPalette->LoadPalette();
		break;
	case ID_BTNOPENTILES: 
		if (sclPalette->GetIsLoaded() == false) {
			wxMessageBox("Load the palette first!", "Notice", wxOK, this);
			break;
		}
		sclTiles->LoadTiles();
		break;
	}
}

MyFrame::MyFrame(wxWindow *parent)
	: wxFrame(parent, -1, "SFC Tile Map Editor")
{
	wxBoxSizer *sizer1 = new wxBoxSizer(wxHORIZONTAL);

		wxPanel *panel1 = new wxPanel(this, -1, wxDefaultPosition, wxSize(150, 140));
		panel1->SetBackgroundColour(*wxGREEN);
			wxBoxSizer *sizer2 = new wxBoxSizer(wxVERTICAL);

				wxBoxSizer *sizer3 = new wxBoxSizer(wxHORIZONTAL);
					sclTiles = new ChrPage(panel1, ID_SCLTILES, wxDefaultPosition, wxSize(150, 80), wxALWAYS_SHOW_SB);
					sclTiles->SetBackgroundColour(*wxBLUE);
					btnOpenTiles = new wxButton(panel1, ID_BTNOPENTILES, "O", wxDefaultPosition, wxSize(30, 30));
				sizer3->Add(sclTiles, 1, wxEXPAND | wxALL);
				sizer3->Add(btnOpenTiles, 0);

			sizer2->Add(sizer3, 7, wxEXPAND | wxALL, 10);

				wxBoxSizer *sizer6 = new wxBoxSizer(wxHORIZONTAL);
					wxStaticText *stattext = new wxStaticText(panel1, -1, "Graphics: ");
				sizer6->Add(stattext);
					wxArrayString string;
					string.Add("2bpp (2 colors)");
					string.Add("3bpp (8 colors, left half)");
					string.Add("3bpp (8 colors, right half)");
					string.Add("4bpp (16 colors)");
					string.Add("8bpp (256 colors)");
					string.Add("mode 7 (256 colors)");
					string.Add("16bpp (direct color)");
					cmbBitSize = new wxComboBox(panel1, ID_CMBBITSIZE, "", wxDefaultPosition, wxDefaultSize, string, wxCB_READONLY);
				sizer6->Add(cmbBitSize);

			sizer2->Add(sizer6);

				wxBoxSizer *sizer4 = new wxBoxSizer(wxHORIZONTAL);
				wxBoxSizer *sizer5 = new wxBoxSizer(wxVERTICAL);
							sclPalette = new PaletteWindow(panel1, ID_SCLPALETTE, wxDefaultPosition, wxSize(150, 60), wxALWAYS_SHOW_SB);
							sclPalette->SetBackgroundColour(*wxLIGHT_GREY);
						sizer4->Add(sclPalette, 1, wxEXPAND | wxALL);
							btnOpenPalette = new wxButton(panel1, ID_BTNOPENPALETTE, "O", wxDefaultPosition, wxSize(30, 30));
							btnOpenPalette24 = new wxButton(panel1, ID_BTNOPENPALETTE24, "2", wxDefaultPosition, wxSize(30, 30));
					sizer5->Add(btnOpenPalette, 0);
					sizer5->Add(btnOpenPalette24, 0);
					sizer4->Add(sizer5, 0);

			sizer2->Add(sizer4, 5, wxEXPAND | wxALL, 10);
	
		panel1->SetSizerAndFit(sizer2);
	sizer1->Add(panel1, 1, wxEXPAND | wxALL, 10);

		sclTileMap = new wxScrolled<wxWindow>(this, ID_SCLTILEMAP, wxDefaultPosition, wxSize(150, 140), wxALWAYS_SHOW_SB);
		sclTileMap->SetBackgroundColour(*wxBLUE);
		btnSelection = new wxToggleButton(this, -1, "S", wxDefaultPosition, wxSize(30, 30));

	sizer1->Add(btnSelection, 0, wxCENTER, 10);
	sizer1->Add(sclTileMap, 1, wxEXPAND | wxALL, 10);

	SetSizerAndFit(sizer1);

	sclTiles->SetPaletteWindowPtr(sclPalette);

	mbrMain = new wxMenuBar;
	mnuFile = new wxMenu;
	mnuFile->Append(wxID_NEW, "&New");
	mnuFile->Append(wxID_OPEN, "&Open");
	mnuFile->Append(wxID_SAVE, "&Save");
	mnuFile->Append(wxID_SAVEAS, "Save &As");
	mnuFile->AppendSeparator();
	mnuFile->Append(wxID_EXIT, "E&xit");
	mbrMain->Append(mnuFile, "&File");
	SetMenuBar(mbrMain);
}
