//SFC graphics functions (source) 
//by beneficii

#include "Graphics.h"

std::size_t GetBitSizeInBytes(int bitsize) {
	switch (bitsize) {
	case BITSIZE_2BPP:
		return 16;
	case BITSIZE_3BPP_LEFT:
	case BITSIZE_3BPP_RIGHT:
		return 24;
	case BITSIZE_4BPP:
		return 32;
	case BITSIZE_8BPP:
	case BITSIZE_MODE_7:
		return 64;
	case BITSIZE_DIR:
		return 128;
	}
	return 16;
}

void SfcColorToRGB(int color, int *red, int *green, int *blue) {
	*red = (color & 0x1f) << 3;
	*green = (color & 0x3e0) >> 2;
	*blue = (color & 0x7c00) >> 7;
}

wxColour SfcColorToRGB(int color) {
	return wxColour((color & 0x1f) << 3, (color & 0x3e0) >> 2, (color & 0x7c00) >> 7);
}

int RGBToSfcColor(int red, int green, int blue) {
	int retval = (red & 0xf8) >> 3;
	retval |= (green & 0xf8) << 2;
	retval |= (blue & 0xf8) << 7;
	return retval;
}

void AllRGBToSfcColor(unsigned char *write, const unsigned char *read, int num_colors) {
	for (int i = 0; i < num_colors; i++) {
		int color = RGBToSfcColor(read[i * 3], read[i * 3 + 1], read[i * 3 + 2]);
		write[i * 2] = (unsigned char)(color & 0xff);
		write[i * 2 + 1] = (unsigned char)((color & 0x7f00) >> 8);
	}
}

static constexpr int GetBit(unsigned char tilebyte, int bitmask, int x, int offset) {
	return (tilebyte & bitmask) >> (7 - x) << offset;
}

void DrawTile_2bpp(const unsigned char *tiledata, PointDrawType function) {
	for (int y = 0; y < 8; ++y) {
		for (int x = 0; x < 8; ++x) {
			int bitmask = 0x80 >> x;
			int color = GetBit(tiledata[y * 2], bitmask, x, 0);
			color |= GetBit(tiledata[y * 2 + 1], bitmask, x, 1);
			function(x, y, color);
		}
	}
}

void DrawTile_3bpp(const unsigned char *tiledata, PointDrawType function) {
	for (int y = 0; y < 8; ++y) {
		for (int x = 0; x < 8; ++x) {
			int bitmask = 0x80 >> x;
			int color = GetBit(tiledata[y * 2], bitmask, x, 0);
			color |= GetBit(tiledata[y * 2 + 1], bitmask, x, 1);
			color |= GetBit(tiledata[y + 16], bitmask, x, 2);
			function(x, y, color);
		}
	}
}

void DrawTile_4bpp(const unsigned char *tiledata, PointDrawType function) {
	for (int y = 0; y < 8; ++y) {
		for (int x = 0; x < 8; ++x) {
			int bitmask = 0x80 >> x;
			int color = GetBit(tiledata[y * 2], bitmask, x, 0);
			color |= GetBit(tiledata[y * 2 + 1], bitmask, x, 1);
			color |= GetBit(tiledata[y * 2 + 16], bitmask, x, 2);
			color |= GetBit(tiledata[y * 2 + 17], bitmask, x, 3);
			function(x, y, color);
		}
	}
}

void DrawTile_8bpp(const unsigned char *tiledata, PointDrawType function) {
	for (int y = 0; y < 8; ++y) {
		for (int x = 0; x < 8; ++x) {
			int bitmask = 0x80 >> x;
			int color = GetBit(tiledata[y * 2], bitmask, x, 0);
			color |= GetBit(tiledata[y * 2 + 1], bitmask, x, 1);
			color |= GetBit(tiledata[y * 2 + 16], bitmask, x, 2);
			color |= GetBit(tiledata[y * 2 + 17], bitmask, x, 3);
			color |= GetBit(tiledata[y * 2 + 32], bitmask, x, 4);
			color |= GetBit(tiledata[y * 2 + 33], bitmask, x, 5);
			color |= GetBit(tiledata[y * 2 + 48], bitmask, x, 6);
			color |= GetBit(tiledata[y * 2 + 49], bitmask, x, 7);
			function(x, y, color);
		}
	}
}

void DrawTile_Mode7(const unsigned char *tiledata, PointDrawType function) {
	for (int y = 0; y < 8; ++y) {
		for (int x = 0; x < 8; ++x) {
			function(x, y, (int)tiledata[x + 8 * y]);
		}
	}
}

void DrawTile_DirectColor(const unsigned char *tiledata, PointDrawType function) {
	for (int y = 0; y < 8; ++y) {
		for (int x = 0; x < 8; ++x) {
			int color = tiledata[(x + 8 * y) * 2];
			color |= tiledata[1 + (x + 8 * y) * 2] << 8;
			function(x, y, color);
		}
	}
}

TileMap_Tile GetTileMap_Tile(int data) {
	TileMap_Tile tile;
	tile.verticalFlip = data & 0x8000 ? true : false;
	tile.horizontalFlip = data & 0x4000 ? true : false;
	tile.priorityBit = data & 0x2000 ? true : false;
	tile.paletteNumber = (data & 0x1c00) >> 10;
	tile.tileNumber = data & 0x3ff;
	return tile;
}

int GetTileMap_TileData(const TileMap_Tile &tile) {
	int retval = tile.verticalFlip ? 0x8000 : 0;
	retval |= tile.horizontalFlip ? 0x4000 : 0;
	retval |= tile.priorityBit ? 0x2000 : 0;
	retval |= (tile.paletteNumber & 3) << 10;
	retval |= tile.tileNumber & 0x3ff;
	return retval;
}
